%% PolynomialSensFitting.m
%
% Function that fits a given sensitivity profile by a 2D polynomial of
% given degree.
%
% INPUT:    * sens: sensitivity map
%           * D: degree of polynomial
%           * FOV: vector defining spatial dimensions
%           * support (optional): support for fitting
%
% OUTPUT:   * s: (N,1) vector of parameters, s.t
%              s(1)+s(2)*x1+s(3)*x2+s(4)x1^2+s(5)*x1*x2+....+s(N)*x2^D
%           * nrmse: normalized root mean square error
%           * ser: signal to error ratio of the approximation
%           * maxerror: maximal error of approximation inside the support
%
% Note: In 2D we have N = (D+1)*(D+2)/2.
%
% Matthieu Guerquin-Kern, Biomedical Imaging Group / EPF Lausanne,
% 30-10-2009 (dd-mm-yyyy)

function [s,nrmse,ser,maxerror] = PolynomialSensFitting(sens,D,FOV,support)

res = size(sens);
if nargin<4
    support = ones(res);
end

if isempty(FOV)
    FOV = 1;
end

if numel(FOV)==1
    FOV = FOV*[1,1];
end

ind = find(support);    % indexes corresponding to the support

% Define the points inside the support
[X1,X2] = GenerateFullCart2DGrid(res);
R = [X1(ind)'*FOV(1)/res(1);X2(ind)'*FOV(2)/res(2)];
clear X1 X2;

% Matrix related to the points of the support
M = Polynomial2DMatrix(R,D);

% Compute the parameters
s = M\sens(ind);

if nargout>1
    residue = sens(ind)-M*s;
    mse = residue'*residue/numel(residue);
    nrmse = sqrt(mse)/(max(abs(sens(ind))-min(abs(sens(ind)))));
    ser = (sens(ind)'*sens(ind))/mse/numel(residue);
    maxerror = max(abs(residue))/(max(abs(sens(ind))-min(abs(sens(ind)))));
end